@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight.
-   Copyright 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002, 2003, 2004
    Free Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
@@ -20,8 +20,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "inferior.h"
@@ -64,6 +64,10 @@
 #include "dis-asm.h"
 #include "gdbcmd.h"
 
+#ifdef __CYGWIN__
+#include <sys/cygwin.h>		/* for cygwin_conv_to_full_win32_path */
+#endif
+
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>		/* for isprint() */
 #endif
@@ -2825,7 +2829,7 @@ gdb_path_conv (ClientData clientData, Tcl_Interp *interp,
   {
     char pathname[256], *ptr;
 
-    cygwin32_conv_to_full_win32_path (Tcl_GetStringFromObj (objv[1], NULL),
+    cygwin_conv_to_full_win32_path (Tcl_GetStringFromObj (objv[1], NULL),
 				      pathname);
     for (ptr = pathname; *ptr; ptr++)
       {
