@@ -1,5 +1,5 @@
 /* Startup code for Insight
-   Copyright 1994, 1995, 1996, 1997, 1998, 2001, 2002, 2003
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 2001, 2002, 2003, 2004
    Free Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
@@ -18,8 +18,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "inferior.h"
@@ -31,6 +31,7 @@
 #include "version.h"
 #include "top.h"
 #include "annotate.h"
+#include "exceptions.h"
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 #define WIN32_LEAN_AND_MEAN
@@ -417,10 +418,6 @@ gdbtk_init (void)
 	      set env(IWIDGETS_LIBRARY)\
                      [file join $srcDir itcl iwidgets generic]\n\
 	  }\n\
-\
-	  if {![info exists env(TIX_LIBRARY)]} {\n\
-	      set env(TIX_LIBRARY) [file join $srcDir tix library]\n\
-	  }\n\
 \
 	  if {![info exists env(GDBTK_LIBRARY)]} {\n\
 	      set env(GDBTK_LIBRARY) [file join $srcDir gdb gdbtk library]\n\
@@ -633,7 +630,13 @@ gdbtk_find_main";
 	 If GDB wasn't started from the DOS prompt, the user won't
 	 get to see the failure reason.  */
       MessageBox (NULL, msg, NULL, MB_OK | MB_ICONERROR | MB_TASKMODAL);
-      throw_exception (RETURN_ERROR);
+      {
+        struct gdb_exception e;
+        e.reason  = RETURN_ERROR;
+        e.error   = GENERIC_ERROR;
+        e.message = msg;
+        throw_exception (e);
+      }
 #else
       /* FIXME: cagney/2002-04-17: Wonder what the lifetime of
 	 ``msg'' is - does it need a cleanup?  */
@@ -658,7 +661,7 @@ static void
 gdbtk_init_1 (char *arg0)
 {
   argv0 = arg0;
-  init_ui_hook = NULL;
+  deprecated_init_ui_hook = NULL;
 }
 
 /* gdbtk_test is used in main.c to validate the -tclcommand option to
@@ -684,7 +687,7 @@ _initialize_gdbtk ()
      if "interpreter_p" is set to "insight" to know if
      insight is GOING to run. */
   if (strcmp (interpreter_p, "insight") == 0)
-    init_ui_hook = gdbtk_init_1;
+    deprecated_init_ui_hook = gdbtk_init_1;
 #ifdef __CYGWIN__
   else
     {
