@@ -1,5 +1,5 @@
 # Local preferences functions for Insight.
-# Copyright 1997, 1998, 1999, 2002, 2003 Red Hat
+# Copyright (C) 1997, 1998, 1999, 2002, 2003, 2004 Red Hat
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License (GPL) as published by
@@ -50,7 +50,8 @@ proc pref_read {} {
     set prefs_init_filename ".gdbtkinit"
   }
 
-  if {!$GDBStartup(inhibit_prefs)} {
+  if {!$GDBStartup(inhibit_prefs) \
+      || [info exists env(INSIGHT_FORCE_READ_PREFERENCES)]} {
     set file_opened 0
     if {[file exists $prefs_init_filename]} {
       if {[catch {open $prefs_init_filename r} fd]} {
@@ -142,7 +143,17 @@ proc pref_read {} {
   }
 
   # finally set colors, from system if possible
-  pref_set_colors $home
+  if {[pref get gdb/use_color_schemes] != "1"} {
+    pref_set_colors $home
+  } else {
+    global Colors
+    # These colors are the same for all schemes
+    set Colors(textfg) black
+    set Colors(fg) black
+    set Colors(sbg) \#4c59a5
+    set Colors(sfg) white
+    set_bg_colors
+  }
 }
 
 # ------------------------------------------------------------------
@@ -199,7 +210,7 @@ proc pref_save {{win {}}} {
     # FIXME: this is broken.  We should discover the list
     # dynamically.
     lappend secs load console src reg stack locals watch bp search \
-      process geometry help browser kod window session mem
+      process geometry help browser kod window session mem bg ipc
 
     foreach section $secs {
       puts $fd "\[$section\]"
@@ -405,8 +416,47 @@ proc pref_set_defaults {} {
   
   # External editor.
   pref define gdb/editor ""
+
+  # background colors
+  set ::gdb_bg_num 0
+  pref define gdb/use_color_schemes	1
+  pref define gdb/bg/0	\#ffffff
+  pref define gdb/bg/1	\#ffffd0
+  pref define gdb/bg/2	\#ffd0ff
+  pref define gdb/bg/3	\#ffd0d0
+  pref define gdb/bg/4	\#d0ffff
+  pref define gdb/bg/5	\#d0ffd0
+  pref define gdb/bg/6	\#d0d0ff
+  pref define gdb/bg/7	\#d0d0d0
+  pref define gdb/bg/8	\#ffffb0
+  pref define gdb/bg/9	\#ffb0ff
+  pref define gdb/bg/10	\#ffb0b0
+  pref define gdb/bg/11	\#b0ffff
+  pref define gdb/bg/12	\#b0ffb0
+  pref define gdb/bg/13	\#b0b0ff
+  pref define gdb/bg/14	\#b0b0b0
+  pref define gdb/bg/15	\#d0b0d0
+
+  # IPC prefs
+  # set prefs based on GDB version?
+  #set vers [lindex [split [lindex [split [gdb_cmd "show version"]] end-1 ] \"] 1]
+  pref define gdb/ipc/enabled	1
+  pref define gdb/ipc/port		9909
+  pref define gdb/ipc/stop_button	1
+  pref define gdb/ipc/step_button	1
+  pref define gdb/ipc/cont_button	1
+  pref define gdb/ipc/run_button	1
+  pref define gdb/ipc/exit		1
 }
 
+
+##########################################################################
+#
+# Everything below this point is code to try to determine the current OS
+# color scheme and use that.  It mostly works, but is not very compatible 
+# with the use of multiple color schemes for different instances of Insight.
+#
+##########################################################################
 proc pref_set_colors {home} {
   # set color palette
   
@@ -674,6 +724,7 @@ proc pref_set_option_db {makebg} {
   set Colors(change) "green"
 
   option add *background $Colors(bg)
+  option add *buttonBackground $Colors(bg)
   option add *Text*background $Colors(textbg)
   option add *Entry*background $Colors(textbg)
   option add *foreground $Colors(fg)
@@ -698,11 +749,8 @@ proc pref_set_option_db {makebg} {
   }
   
   if {$makebg} {
-    # compute a slightly darker background color
-    # and use for activeBackground and troughColor
-    set bg2 [winfo rgb . $Colors(bg)]
-    set dbg [format #%02x%02x%02x [expr {(9*[lindex $bg2 0])/2560}] \
-	       [expr {(9*[lindex $bg2 1])/2560}] [expr {(9*[lindex $bg2 2])/2560}]]
+    # calculate trough and activebackground as 90% of background
+    set dbg [recolor $::Colors(bg) 90]
     option add *activeBackground $dbg
     option add *troughColor $dbg
   }
